/*
  Cat graphing and email program
  Language: Processing

  Reads in a string of characters until it gets a linefeed (ASCII 10). 
  Then converts the string into a number. Then graphs it. If the 
  number has changed significantly, and there hasn't been a big 
  change in more than a minute, the program calls a PHP script 
  to send an email message.
*/
import processing.serial.*;
import processing.net.*;  // gives you access to the net library


int linefeed = 10;      // linefeed in ASCII
Serial myPort;          // The serial port
int sensorValue = 0;    // the value from the sensor
int graphPosition = 0;  // the horizontal position of the latest
                        // line to be drawn on the graph

int prevSensorValue = 0;    // the previous sensor reading
boolean catOnMat = false;   // whether or not the cat's on the mat
int threshold = 330;        // above this number, the cat is on the mat.

int timeThreshold = 1;    // minimum number of minutes between emails
int timeLastSent[] = {
  hour(), minute() };      // the time that your last message was sent

// HTTP client variables:
Client client;              // a new net client
boolean requestInProgress = false;  //  whether a net request is in progress
String responseString = "";     // string of text received by client


void setup() {
  size(400,300);
  // List all the available serial ports
  println(Serial.list());

  // I know that the first port in the serial list on my mac
  // is always my  Arduino, so I open Serial.list()[0].
  // Open whatever port is the one you're using.
 // myPort = new Serial(this, Serial.list()[0], 9600);
 myPort = new Serial(this, Serial.list()[0], 9600);
  // read bytes into a buffer until you get a linefeed (ASCII 10):
  myPort.bufferUntil(linefeed);
  println(hour() + ":" + minute());
}

void draw() {

  if (sensorValue > threshold ) {
    // if the last reading was less than the threshold,
    // then the cat just got on the mat.
    if (prevSensorValue <= threshold) {
      delay(100);
      if (sensorValue > threshold) {
        catOnMat = true;
        sendMail();
      }
    } 
  } 
  else {
    // if the sensor value is less than the threshold,
    // and the previous value was greater, then the cat
    // just left the mat
    if (prevSensorValue >= threshold) {
      catOnMat = false;
    }    
  }
  // save the sensor value as the previous value
  // so you can take new readings:
  prevSensorValue = sensorValue;

  if (requestInProgress == true) {
    checkNetClient(); 
  }
}

// serialEvent  method is run automatically by the Processing applet
// whenever the buffer reaches the  byte value set in the bufferUntil() 
// method in the setup():

void serialEvent(Serial myPort) { 
  // read the serial buffer:
  String myString = myPort.readStringUntil(linefeed);
  // if you got any bytes other than the linefeed:
  if (myString != null) {
    //trim off the carriage return and convert the string to an integer:
    sensorValue = int(trim(myString)) /2 -100;
    // print it:
    // println(sensorValue);
    drawGraph();
  }
} 

void drawGraph() {
  int lineHeight = sensorValue /2;
  // draw the line:
  if (catOnMat) {
    // draw green:
    stroke(0,255,0);
  } 
  else {
    // draw red:
    stroke(255,0,0); 
  }
  line(graphPosition, height, graphPosition, height - lineHeight);
  // at the edge of the screen, go back to the beginning:
  if (graphPosition >= width) {
    graphPosition = 0;
    background(0); 
  } 
  else {
    graphPosition++;
  }
}

void sendMail() {
  // calculate the current time in minutes:
  int[] presentTime = { 
    hour(), minute()              };

  // print the current time and the last time you sent a message:
  print(sensorValue + "\t");
  print( presentTime[0] + ":" + presentTime[1] +"\t");
  println(timeLastSent[0] + ":" + timeLastSent[1]);


  // if you're still in the same hour as the last message,
  // then make sure at least the minimum number of minutes has passed:
  if (presentTime[0] == timeLastSent[0]) {
    if (presentTime[1] - timeLastSent[1] >= timeThreshold) {
      println("This is where you'd send a mail."); 
      makeHTTPCall();
      // take note of the time this message was sent:
      timeLastSent[0] = hour();
      timeLastSent[1] = minute();  
    }
  }

  // if the hour has changed since the last message,
  // then the difference in minutes is a bit more complex.
  // Use !+ rather than > to make sure the shift 
  // from 23:59 to 0:00 is covered as well:
  if (presentTime[0] != timeLastSent[0]) {
    // calculate the difference in minutes:
    int minuteDifference = (60 - timeLastSent[1]) + presentTime[1];

    if (minuteDifference >= timeThreshold) {
      println("This is where you'd send a mail."); 
      makeHTTPCall();
      // take note of the time this message was sent:
      timeLastSent[0] = hour();
      timeLastSent[1] = minute();  
    }
  }
}

void makeHTTPCall() {
  if (requestInProgress == false) {
    // Open a connection to the host:
    client = new Client(this, "example.com", 80);

    // form the request string:
    String requestString = "/cat-script.php?sensorValue=" + sensorValue;
    println(requestString);
    // Send the HTTP GET request:
    client.write("GET  " + requestString + " HTTP/1.1\n");
    client.write("HOST: example.com\n\n");
    // note that you've got a request in progress:
    requestInProgress = true;
  }
}

void checkNetClient() {
  // available() returns how many bytes have been received by the client:
  if (client.available() > 0) {
    // read a byte, convert it to a character, and add it to the string:
    responseString +=char(client.read());    

    // add to a line of |'s on the screen (crude progress bar):
    print("|");
  } 
  // if there's no bytes available, either the response hasn't 
  // started yet, or it's done:
  else { 
    // if responseString is longer than 0 bytes, the response has started:
    if(responseString.length() > 0 )  {
      // you've got some bytes, but now there's no more to read. Stop:
      if(requestInProgress == true) {
        // print the response:
        println(responseString);
        // note that the request is over:
        requestInProgress = false;
        // reset the string  for future requests:
        responseString = "";
      }
    }
  } 
}
