/*
  BlueRadios Master Connection
  Language: Wiring/Arduino

  This program assumes that the microcontroller is connected 
  to a BlueRadios bluetooth radio, and that the radio is in master mode.
  When the program starts, it releases the CTSpin pin 
  of the radio, so the radio can send data to the microcontroller.  
  Then it  sends a connect message and listens.  If more than 
  5 seconds passes, it attempts to connect again.  If it 
  receives a comma, which only appears in the 
  CONNECT,<address> string, it assumes the radio is connected and 
  starts sending data. If it receives an S, it assumes 
  the radio is disconnected and stops sending.
*/
#define sensorPin 0         // input sensor
#define txLed 2             // LED to indicate outgoing data
#define rxLed 3             // LED to indicate incoming data
#define CTSpin 4            // Clear-to-send pin
#define analogLed 9         // LED that will change brightness with
                            // incoming value
#define threshold 10        // how much change you need to see on the
                            // sensor before sending

byte lastSensorReading = 0;  // previous state of the pot       
long lastConnectTry;         // milliseconds elapsed since the last
                             // connection attempt
long connectTimeout = 5000;  // milliseconds to wait between 
                             // connection attempts
int inByte= -1;              // incoming byte from serial RX
char inString[6];            // string for incoming serial data
int stringPos = 0;           // string index counter

// address of the remote BT radio. Replace with the address 
// of your remote radio
char remoteAddress[13] = "112233445566";  

byte connected = false;      // whether you're connected or not

void setup() {
  // configure serial communications:
  Serial.begin(9600);      

  // configure output pins:
  pinMode(txLed, OUTPUT);
  pinMode(rxLed, OUTPUT);
  pinMode (analogLed, OUTPUT);
  pinMode(CTSpin, OUTPUT);

  // set CTS low so BlueSMiRF can send you serial data:
  digitalWrite(CTSpin, LOW);

  // Attempt a connection:
  BTConnect();

  // blink the tx LED to say you're done with setup:
  blink(3);

}

void BTConnect() {
  Serial.print("+++\r");
  delay(250);
  Serial.print("ATDH\r");
  Serial.print("ATDM");
  Serial.print(remoteAddress);
  Serial.print(",1101\r");
}

int readSensor() {
  int message = 0;
  // read the sensor:
  int sensorReading = analogRead(sensorPin);

  // look for a change from the last reading
  // that's greater than the threshold:
  if (abs(sensorReading - lastSensorReading) > threshold) {
    message = sensorReading/4;
    lastSensorReading = sensorReading;
  } 
  return message;
}

void blink(int howManyTimes) {
  for (int i=0; i< howManyTimes; i++) {
    digitalWrite(txLed, HIGH);
    delay(200);
    digitalWrite(txLed, LOW);
    delay(200);  
  }
}

void loop() {
  if (Serial.available() > 0) {
    // signal that there's incoming data using the rx LED:
    digitalWrite(rxLed, HIGH);
    // do something with the incoming byte:
    handleSerial();
    // turn the rx LED off.
    digitalWrite(rxLed, LOW);
  } 

  // if you're not connected and 5 seconds have passed in that state,
  // make an attempt to connect to the other radio:
  if (!connected && millis() - lastConnectTry > connectTimeout) {
    BTConnect();
    lastConnectTry = millis();
  }
}

void handleSerial() {
  inByte = Serial.read();
  delay(2);
  // comma comes only in the CONNECT,<address> message:
  if (inByte == ',') {
    // send an initial message:
    sendData();
    // update the connection status:
    connected = true; 
  }

  //S comes only in the DISCONNECT message:
  if (inByte == 'S') {
    // turn off the analog LED:
    analogWrite(analogLed, 0);
    connected = false; 
  }
  //R comes only in the NO CARRIER and NO ANSWER messages:
  if (inByte == 'R') {
    // turn off the analog LED:
    analogWrite(analogLed, 0);
    connected = false; 
  }

  if (connected) {
    // save only ASCII numeric characters (ASCII 0 - 9):
    if ((inByte >= '0') && (inByte <= '9')){
      inString[stringPos] = inByte;
      stringPos++;
    }
    // if you get an asterisk, it's the end of a string:
    if (inByte == '*') {
      // convert the string to a number:
      int brightness = atoi(inString);
      // set the analog output LED:
      analogWrite(analogLed, brightness);

      // put zeroes in the array
      for (int c = 0; c < stringPos; c++) {
        inString[c] = 0;
      } 
      // reset the string pointer:
      stringPos = 0;
      // Since this  byte (*) is the end of an incoming string, 
      // send out your reading in response:
      sendData();
    }
  }
}

void sendData() {
  // indicate that we're sending using the tx LED:
  digitalWrite(txLed, HIGH);
  Serial.print(readSensor(), DEC);
  // string termination:
  Serial.print("*"); 
  // turn off the tx LED:
  digitalWrite(txLed, LOW);
}
