/*
  Color Recognition Using a Webcam
  Language: Processing

  Reads an image from a camera and looks for a blob 
  of a particular color.  Click on a color in the image 
  to choose the color to track.
*/

// import the Processing video library:
import processing.video.*;

Capture myCamera;                      // instance of the Capture class 
color targetColor = color(255,0,0);    // the color you're looking for
int[] matchingPixel = new int[2];

void setup()
{
  // set window size:
  size(320, 240);

  // List all available capture devices. Macintoshes generally
  // identify three cameras, and the first or second is the built-in iSight
  // of the laptops.  Windows machines will need a webcam installed before
  // you can run this program. 
  println(Capture.list());

  // capture from the second device in the list (in my case, my iSight)
  // change this to match your own camera:
  String myCam = Capture.list()[1];
  myCamera = new Capture(this, width, height, myCam, 30);
}

void draw(){
  // draw the current frame of the camera on the screen:
  image(myCamera, 0, 0);
  // draw a dot at the matchingPixel:
  fill(targetColor);
  ellipse(matchingPixel[0], matchingPixel[1], 10, 10);    
}


/* when the mouse is clicked, capture the 
 color of the pixel at the mouse location
 to use as the tracking color
 */
void mouseReleased() {
  // get the color of the mouse position's pixel:
  targetColor = myCamera.pixels[mouseY*width+mouseX];
  matchingPixel[0] = mouseX;
  matchingPixel[1] = mouseY;
}

/*
  Take the target color  and look for a pixel matching the color 
 in the camera image:
 */
void captureEvent(Capture myCamera) {
 
  // intialize the smallest acceptable color difference:
  float smallestDifference = 1000.0;     
  // read the myCamera and update the pixel array:
  myCamera.read();
  // scan over the pixels  to look for a pixel
  // that matches the target color:
  for(int row=0; row<height; row++) { 
    for(int column=0; column<width; column++) { //for each column
      //get the color of this pixel
      //find pixel in linear array using formula: pos = row*rowWidth+column
      color pixelColor = myCamera.pixels[row*width+column];

      // determine the difference between this pixel's color
      // and the target color:
      float diff = abs(red(targetColor) - red(pixelColor)) + abs(green(targetColor) - green(pixelColor)) + abs(blue(targetColor) - blue(pixelColor))/3;

      // if this is closest to our target color, take note of it:
      if (diff<= smallestDifference){ 
        smallestDifference = diff;
        // save the position:
        matchingPixel[0] = row; 
        matchingPixel[1] = column;
      } 
    }
  }  
}

